///////////////////////////////////////////////////////////////////////////////////////////////
//  LCD display Source File
//	writes AD data to LCD screen
///////////////////////////////////////////////////////////////////////////////////////////////

#include <p24FJ192GA110.h>
#include "LCDdisp.h"
#include <string.h>

// initialization of the LCD
////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////

void LCDinit( void)
{
    // PMP initialization 
    PMCON = 0x83BF;             	// Enable the PMP
    PMMODE = 0x3FF;             	// Master Mode 1
	//_PTEN0 = 0x0001;            // PMA0 enabled
	PMAEN = 0x0001;			// same as above enables PMA0
    
    // init TMR1
    T1CON = 0x8030;             	// Fosc/2, prescaled 1:256, 16us/tick

    // wait for >30ms
    TMR1 = 0; 
	while( TMR1<2000);		// 2000 x 16us = 32ms   
    
    //initiate the HD44780 display 8-bit init sequence
    PMADDR = LCDCMD;            	// command register
    PMDATA = 0b00111000;        	// 8-bit interface, 2 lines, 5x7
    TMR1 = 0; 
	while( TMR1 < 3);   		// 3 x 16us = 48us   
    
    PMDATA = 0b00001100;        	// display ON, cursor off, blink off
    TMR1 = 0; 
	while( TMR1 < 3);   		// 3 x 16us = 48us   
    
    PMDATA = 0b00000001;        	// clear display
    TMR1 = 0; 
	while( TMR1 < 100); 		// 100 x 16us = 1.6ms   
    
    PMDATA = 0b00000110;        	// increment cursor, no shift
    TMR1 = 0; 
	while( TMR1 < 100); 		// 100 x 16us = 1.6ms   
} // initLCD

// reading the LCD
////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////

char readLCD( int addr)
{
    int dummy;
    while( PMMODEbits.BUSY);    	// wait for PMP to be available
    PMADDR = addr;              	// select the command address
    dummy = PMDATA;            	// initiate a read cycle, dummy read
    while( PMMODEbits.BUSY);    	// wait for PMP to be available
    return( PMDATA);            	// read the status register
    
} // readLCD

// reading the LCD
////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////

void writeLCD( int addr, char c)    
{
    while( busyLCD());
    while( PMMODEbits.BUSY);    	// wait for PMP to be available
    PMADDR = addr;
    PMDATA = c;
} // writeLCD

// put characters in the LCD
////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////

void putsLCD( char *s)
{
    while( *s) putLCD( *s++);
} //putsLCD



// add a decimal into three digit char string for tempx
/////////////////////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////////////////////

void adddec( char *tempx)
{
	char buff;
	if( strlen(tempx) == 2)
		{
			buff = tempx[1];		// put in '.' at 2nd charater pos and move up last character
			tempx[1] = '.';
			tempx[2] = buff;
			tempx[3] = '\0';
		}
	else if( strlen(tempx) == 4)		// put in '.' at 4th character pos and move up last charater
		{
			buff = tempx[3];
			tempx[3] = '.';
			tempx[4] = buff;
			tempx[5] = '\0';
		}
	else if( strlen(tempx) == 3)		// put in '.' at 3rd character pos and move up last charater
		{
			buff = tempx[2];
			tempx[2] = '.';
			tempx[3] = buff;
			tempx[4] = '\0';
		}
	else if( strlen(tempx) == 1)		// put in '.' at 3rd character pos and move up last charater
		{
			buff = tempx[0];
			tempx[0] = '.';
			tempx[1] = buff;
			tempx[2] = '\0';
		}
}





void addneg( char *bufferX)
{
	int i;
	char buffstr[12];
	buffstr[0] = '-';
	for( i = 0; i < 11; i++)
	{
		buffstr[i+1] = bufferX[i];
	}
	for( i = 0; i < 12; i++)
		bufferX[i] = buffstr[i];
}
