////////////////////////////////////////////////////////////////////////////////////////////
//  main() for pic24FJ192GA110 weather station
//	Storey, Tony
//	7 June 2010
//	includes WDT interrupt for leaving idle mode and CN interrupt for interfacing data
//	UART interrupt for RX and TX information via XBee
//
//
////////////////////////////////////////////////////////////////////////////////////////////


/*************** INCLUDE FILES *************************************
*  standard library, studio, power managment, watchdog timer, ADconv
*
*******************************************************************/
#include <p24FJ192GA110.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "PwrMgnt.h"
#include "wdt.h"
#include "ADconv.h"
#include "LCDdisp.h"
#include "conU2.h"
#include "uart2.h"


/*************** COFIGURATION **************************************
*	Watchdog Timer Disabled/ ICD Pins Select
*	Two Speed Start-up enabled, JTAG disabled/Code Protect/Write Protect
*	Oscillator Selection: HS oscillator with PLL
*	Clock switching and clock monitor both enabled
* 	WatchDog Timer Post scaler set to 1:2048 and windowed WDT off
*	Disable erase/write protect of all memory regions.
********************************************************************/
_CONFIG1( JTAGEN_OFF & GCP_OFF & GWRP_OFF & COE_OFF & FWDTEN_OFF & ICS_PGx2 & WDTPS_PS2048 & WINDIS_OFF) 

_CONFIG2( FCKSM_CSDCMD & OSCIOFNC_OFF & POSCMOD_XT & FNOSC_PRIPLL & IOL1WAY_ON & IESO_ON)

_CONFIG3( WPCFG_WPCFGDIS & WPDIS_WPDIS)		

/************** PROTOTYPES ****************************************
*
*******************************************************************/
void CN_init(void);
extern void InitUART2(void);
void __attribute__((interrupt, no_auto_psv)) _CNInterrupt(void);
void __attribute__ ((interrupt, no_auto_psv)) _U2RXInterrupt(void);
void delay( int time);

/************** INITIALIZE GLOBAL VARS *****************************
*
********************************************************************/
unsigned int _temp16;
unsigned char opt, tempopt;
unsigned int uflag;

#define TRUE	1
#define FALSE	0






int main( void)
{

	

	int j;
	unsigned int a;
	unsigned int i, battlevel, tempBatt, humidlevel, tempHumid, lightlevel, tempLight;
	float tempLux;
	long TRH, SRH;
	unsigned int pinstat = 0;
	int tempC, tempF, TRHint;
	int postempC, postempF;
	char tempstrC[12] = " ";
	char tempstrF[12] = " ";
	char tempstrBatt[12] = " Vdc";
	char bufferHumid[13];
	char bufferBatt[13];
	char bufferC[12];
	char bufferF[12];
	char bufferLight[12];
	char uarttemp[8] = "T";					// string for uart tx of temp
	char uartbatt[6] = "B0";				// string for uart tx of batt volt
	char uarthumid[6] = "H";				// string for uart tx of humid
	char uartlight[7] = "L";				// string for uart tx of Light in LUX

	/*****************************intitializations************************************
	* disable nested interrupts, set timer 1 priority, set period register for interrupt
	* Set prorta as output for testing, timer enabled with prescaler
	*********************************************************************************/
	CN_init();							// initialise Change Notification interrupts 
	LCDinit();							// initialize the LCD

	// I/O REMAP
	//////////////////////////////////////////////////////////////////////////////////////////////
	//
	//////////////////////////////////////////////////////////////////////////////////////////////
	// remap pins before intialising UART2
	// Unlock Registers
	__builtin_write_OSCCONL(OSCCON & 0xbf);
	// Configure Input Functions **********************
	// Assign UART2RX To Pin RP10
	RPINR19bits.U2RXR = 10;
	// Configure Output Functions *********************
	// Assign UART2TX To Pin RP17
	RPOR8bits.RP17R = U2TX_IO;
	// Lock Registers
	__builtin_write_OSCCONL(OSCCON | 0x40);



	InitUART2();						// Initialize UART2 for 115200,8,N,1 TX/RX



	_NSTDIS = 1;						// set 15th bit in INTCON1 register to disable nested interrupts


	

	




	clrscr();							//clear the hyperterminal
	home();							//place cursor in upper right hand corner
//	putsU2( "HELLO");						// test

	//////////////////////////////// MAIN LOOP ///////////////////////////////////////////////////
	//main code
	while(1)
	{
		
		
		
		// ****************** INITIALIZATIONS *************************************************************

		initADC( AINPUTS); 				// initialixe the ADC for the Ecplorer16 analog inputs
		TRISA = 0xff00;					// select PORTA pins as outputs to drive the LEDs
		
	

		//******************************* BATTERY LEVEL AND CONVERSION ************************************

		// sample voltage level from battery on AN5 and take average of 16 samples
		a = 0;
		
		for( j = 0; j < 16; j++)
		{
			a += readADC( BATT);				// read the battery level on AN5
		}	
		
		battlevel = a >> 4;  					// shift by four bits or devide by 16 to get average temp
		tempBatt = battlevel * 33;				// gives Vout = battlevel (ADCreading) * ADC resolution (mV/bit) 
		
		tempBatt = tempBatt / 1024;	

		//For voltage devider with 634k shunted with 1M ohm
		tempBatt = tempBatt * 1632;
		tempBatt = tempBatt / 1000;

		sprintf( bufferBatt, "%d", tempBatt);		// change from int to char
		// ***************** create uart char for transmission ***********************
		uartbatt[2] = bufferBatt[0];
		uartbatt[3] = bufferBatt[1];
		uartbatt[4] = bufferBatt[2];
		uartbatt[5] = '\0';
		
		adddec( bufferBatt);					// put in the decimal place
		strcat( bufferBatt, tempstrBatt); 			// append Vdc on end of char

		//************************** END OF BATTERY LEVEL AND CONVERSION **********************************




		//******************************* TEMPERATURE CONVERSION ******************************************
		

		// sample temperature and take average of 16 samples
	
		a = 0;
		
		for( j = 0; j < 16; j++)
		{
			a += readADC( TSENS);				// read the temp sensor
			Nop();
		}	

		i = a >> 4;  						// shift by four bits or devide by 16 to get average temp
	
		tempC = (i * 32);						// calculate the value of the temperature in degC
		tempC = tempC - 5016; 					// tempC = (Vout-500mV)/(10(mV/C)); Vout = ADC reading*ADC resolution 
		tempC = tempC / 10;
		tempF = tempC * 9;					// calculate the value of the temperature in degF
		tempF = tempF / 5;
		tempF = tempF + 320;
		
		

		if( tempC < 0)						// check for negative tempC
		{
				
			postempC = abs(tempC);				// get rid of neg sign
			sprintf( bufferC, "%d", postempC);		// change from int to char
			// string for transmission via uart and xbee	
			// *************************** UART FORMATTING FOR DATA uarttemp ********************************
			if( tempC <= -100)
			{
				uarttemp[1] = '-';
				uarttemp[2] = bufferC[0];
				uarttemp[3] = bufferC[1];
				uarttemp[4] = bufferC[2];
				uarttemp[5] = '\0';
			}
			else if( tempC <= -10)
			{
				uarttemp[1] = '-';
				uarttemp[2] = '0';
				uarttemp[3] = bufferC[0];
				uarttemp[4] = bufferC[1];
				uarttemp[5] = '\0';	
			}			
			else if( tempC <= -1)
			{
				uarttemp[1] = '-';
				uarttemp[2] = '0';
				uarttemp[3] = '0';
				uarttemp[4] = bufferC[0];
				uarttemp[5] = '\0';	
			}			
			else if( tempC < 0)
			{
				uarttemp[1] = '-';
				uarttemp[2] = '0';
				uarttemp[3] = '0';
				uarttemp[4] = '0';
				uarttemp[5] = '\0';	
			}			
			// ***************** END UART FORMATTING FOR DATA uarttemp  ************************************
			adddec( bufferC);					// add in the decimal place
			addneg( bufferC);					// add in the negative sign
			strcat( bufferC, tempstrC);			// append the deg C on the end of char
		}// end if tempC < 0)

		if( tempF < 0)						// check for negative tempF
		{
			postempF = abs(tempF);				// get rid of neg sign
			sprintf( bufferF, "%d", postempF);	// change from int to char
			adddec( bufferF);					// add in the decimal place
			addneg( bufferF);					// add in the negative sign
			strcat( bufferF, tempstrF);			// append the deg F on the end of char
		}

		if( tempC >= 0)
		{
			sprintf( bufferC, "%d", tempC);		// convert int tempX into char bufferX			
			// ************************** UART DATA FORMATTING FOR uarttemp ********************************
			if( tempC >= 1000)
			{
				uarttemp[1] = bufferC[0];
				uarttemp[2] = bufferC[1];
				uarttemp[3] = bufferC[2];
				uarttemp[4] = bufferC[3];
				uarttemp[5] = '\0';
			}
			else if( tempC >= 100)	
			{
				uarttemp[1] = '0';
				uarttemp[2] = bufferC[0];
				uarttemp[3] = bufferC[1];
				uarttemp[4] = bufferC[2];
				uarttemp[5] = '\0';
			}
			else if( tempC >= 10)
			{
				uarttemp[1] = '0';
				uarttemp[2] = '0';
				uarttemp[3] = bufferC[0];
				uarttemp[4] = bufferC[1];
				uarttemp[5] = '\0';
			}
			else if( tempC >= 1)
			{
				uarttemp[1] = '0';
				uarttemp[2] = '0';
				uarttemp[3] = '0';
				uarttemp[4] = bufferC[0];
				uarttemp[5] = '\0';
			}
			else if( tempC == 0)
			{
				uarttemp[1] = '0';
				uarttemp[2] = '0';
				uarttemp[3] = '0';
				uarttemp[4] = '0';
				uarttemp[5] = '\0';
			}
			// ***************** END OF UART DATA FORMATTING FOR uarttemp *******************************

			adddec( bufferC);					// adds decimal to 2nd index of bufferX char array
			strcat( bufferC, tempstrC);			// format string to add deg X to end of bufferX

		}// end if( tempC >=0)

		

		if( tempF >= 0)
		{
			sprintf( bufferF, "%d", tempF);		// convert int tempX into char bufferX
			adddec( bufferF);					// adds decimal to 2nd index of bufferX char array
			strcat( bufferF, tempstrF);			// format string to add deg X to end of bufferX
		}
		//********************** END OF TEMPERATURE CONVERSION ******************************************





		//********************* HUMIDITY SENSOR AND AD CONVERSION ***************************************
		
		// sample humidity and take average of 16 samples
	
		a = 0;
		
		for( j = 0; j < 16; j++)
		{
			a += readADC( HUMID);				// read the humid sensor
		}	

		humidlevel = a >> 4;  					// shift by four bits or devide by 16 to get average humid

		tempHumid = humidlevel * 33;				// ADC value * ADC resolution 
		tempHumid = tempHumid / 10;				// tempHumid is sensor Vout sampled
	
		TRH = (tempC - 4878);
		TRH = 11017900 / TRH;
		SRH = tempC - 4878;

		SRH = tempBatt * SRH * 1000 / 1793;
		SRH = -72725600 / SRH;
		SRH = SRH * tempHumid;
		SRH = SRH / 100;
		TRH = TRH + SRH;
		TRH = TRH / 100;
		TRHint = TRH;

		

		sprintf( bufferHumid, "%d", TRHint);
		bufferHumid[2] = '%';
		bufferHumid[3] = '\0';


		// ***************** create uart char for transmission ***********************
		uarthumid[1] = bufferHumid[0];
		uarthumid[2] = bufferHumid[1];
		uarthumid[3] = '\0';

		// ****************** end uart char for transmission *************************
		//******************** END HUMIDITY SENSOR AND AD CONVERSION ************************************





		//******************** LIGHT SENSING AD CONVERSION **********************************************
		
		// sample light and take average of 16 samples
	
		a = 0;
		
		for( j = 0; j < 16; j++)
		{
			a += readADC( LIGHT);					// read the light sensor
			Nop();
		}	

		lightlevel = a >> 4;  						// shift by four bits or devide by 16 to get average light ADC value

		tempLight = lightlevel * 33;					// multiply by 3.3 (mV/bit) * 10 
		tempLight = tempLight / 10;					// tempLight is sensor Vout sampled
		
		if( tempLight <= 415)						// choose conversion based on Vout <= 415 is a curve
		{
			tempLux = (tempLight/1000.0 / 10000.0);
			tempLux = tempLux - (1.1/1000000);
			tempLux = tempLux / (.808/1000000);
			tempLux = tempLux*tempLux;
		}
		else if( tempLight > 415)					// choose conversion based on Vout > 415 is a line
		{
			tempLux = tempLight/1000.0 / 10000.0;
			tempLux = tempLux - (23.0/1000000);
			tempLux = tempLux / (.0083/1000000);
		}


		tempLux = (int) (tempLux + .5);				// round tempLux up to an int
		sprintf(bufferLight, "%d", (int) tempLux);		// store tempLux as a char "bufferLight"

		if( tempLux >= 10000)						// do both LCD formatting and UART formatting based on LUX value
		{
			bufferLight[5] = ' ';
			bufferLight[6] = 'L';
			bufferLight[7] = 'u';
			bufferLight[8] = 'x';
			bufferLight[9] = '\0';
			uartlight[1] = bufferLight[0];
			uartlight[2] = bufferLight[1];
			uartlight[3] = bufferLight[2];
			uartlight[4] = bufferLight[3];
			uartlight[5] = bufferLight[4];
			uartlight[6] = '\0';

		}
		else if( tempLux >= 1000)
		{
			bufferLight[4] = ' ';
			bufferLight[5] = 'L';
			bufferLight[6] = 'u';
			bufferLight[7] = 'x';
			bufferLight[8] = '\0';
			uartlight[1] = '0';
			uartlight[2] = bufferLight[0];
			uartlight[3] = bufferLight[1];
			uartlight[4] = bufferLight[2];
			uartlight[5] = bufferLight[3];
			uartlight[6] = '\0';

		}
		else if( tempLux >= 100)
		{		
			bufferLight[3] = ' ';
			bufferLight[4] = 'L';
			bufferLight[5] = 'u';
			bufferLight[6] = 'x';
			bufferLight[7] = '\0';
			uartlight[1] = '0';
			uartlight[2] = '0';
			uartlight[3] = bufferLight[0];
			uartlight[4] = bufferLight[1];
			uartlight[5] = bufferLight[2];
			uartlight[6] = '\0';

		}			
		else if( tempLux >= 10)
		{		
			bufferLight[2] = ' ';
			bufferLight[3] = 'L';
			bufferLight[4] = 'u';
			bufferLight[5] = 'x';
			bufferLight[6] = '\0';
			uartlight[1] = '0';
			uartlight[2] = '0';
			uartlight[3] = '0';
			uartlight[4] = bufferLight[0];
			uartlight[5] = bufferLight[1];
			uartlight[6] = '\0';

		}			
		
		else if( tempLux >= 1)
		{		
			bufferLight[1] = ' ';
			bufferLight[2] = 'L';
			bufferLight[3] = 'u';
			bufferLight[4] = 'x';
			bufferLight[5] = '\0';
			uartlight[1] = '0';
			uartlight[2] = '0';
			uartlight[3] = '0';
			uartlight[4] = '0';
			uartlight[5] = bufferLight[0];
			uartlight[6] = '\0';

		}		
		//******************** END OF LIGHT SENSING AD CONVERSION ***************************************





		//********************** WRITE TO LCD ***********************************************************
		// display info dependent on pin change (RD6/S3, RD7/S6, RA7/S5, RD13/S4)

		_temp16 |= 0xDF3B;
	
		if( _temp16 == 0xFFBF)						// check to see if global variable is one of four pins
			pinstat = _temp16;	//RD6/CN14			// if so set pinstat to _temp16  varaible will not change unless...
		else if( _temp16 == 0xFF7F)					// ...above condition is met
			pinstat = _temp16;	//RD7/CN16
		else if( _temp16 == 0xFFFB)
			pinstat = _temp16;	//RD2/CN51
		else if( _temp16 == 0xDFFF)
			pinstat = _temp16;	//RD13/CN19

		// write battery volt
		if( pinstat == 0xFFBF)
		{
			clrLCD();							// clear the LCD
			LCDinit();							// initialize the LCD
			if( tempBatt <= 34)
			{
				putsLCD( "Low");
			}else
			putsLCD( bufferBatt);					// displays battery voltage if RD6 goes low
			setLCDC(0x40);						// write to bottom row
			putsLCD( "Battery");
	
		}
		//write temp
		else if( pinstat == 0xFF7F)
		{
			clrLCD();							// clear the LCD
			LCDinit();							// initialize the LCD
			putsLCD( bufferC);					// write the tempC to the LCD
			putLCD( 0b11011111);
			putsLCD("C");
			setLCDC(0x40);						// write to bottom row
			putsLCD( bufferF);
			putLCD( 0b11011111);					// write the tempF to the LCD
			putsLCD( "F");
		}
		//write humidity
		else if( pinstat == 0xDFFF)
		{
			clrLCD();
			LCDinit();
			putsLCD( bufferHumid);
			setLCDC(0x40);	
			putsLCD( "Humidity");
		}
		//write light
		else if( pinstat == 0xFFFB)
		{
			clrLCD();
			LCDinit();
			putsLCD( bufferLight);
		}
		//********************** END WRITE TO LCD *******************************************************
 
	


		//********************** UART TX and RX *********************************************************

		if( uflag == 1)
		{
			if( opt == 'R')
			{


				putU2( '4');
				putsU2( uartbatt);				// prompt in hyperterm for testing

				putsU2( uarttemp);

				putsU2( uarthumid);

				putsU2( uartlight);	
				uflag = 0;
				delay( 23000);	

			}
			else if( opt == 'H')
			{

				delay( 30000);
				putU2('*');
				uflag = 0;
				delay( 20000);
			}
			else if( opt == 'T')
			{

				delay( 30000);
				putsU2( "ERROR");
				uflag = 0;
				delay( 20000);
			}
		}
		uflag = 0;
		
		//********************* END UART TX AND RX *****************************************************


		
		//******* Device Enters Sleep mode and wakes up from sleep upon Watchdog timer time out *********
		



		EnableWDT(WDT_ENABLE);						// Enable Watchdog Timer
		U2MODEbits.WAKE = 1;

	
		mPWRMGNT_GotoSleepMode();					// Put the device into Sleep mode
		

		mPWRMGNT_Clear_WDTObit();
		EnableWDT(WDT_DISABLE);             			// Disable WatchDog timer
		RCONbits.SWDTEN = 0;	
		delay( 9000);

	
		//***** END Device Enters Sleep mode and wakes up from sleep upon Watchdog timer time out *******		

	} //main while loop

}//end main


// PIN CHANGE INTERRUPT INITIALIZATION
////////////////////////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////////////////////////////////////
void CN_init(void)
{
	
	TRISDbits.TRISD6 = 1;	// make the port 'RD6' as input
	TRISDbits.TRISD7 = 1;	// make the port 'RD7' as input
	TRISDbits.TRISD13 = 1;	// make the port 'RD13' as input
	TRISDbits.TRISD2 = 1;	// make the port 'RD2' as input
	TRISDbits.TRISD1 = 1;	// make the port 'RD1' as input

	INTCON2bits.INT0EP = 1;	// edge detect 1 = int on neg edge
	CNEN2bits.CN16IE = 1;	// enable pin change interrupt RD7
	CNPU2bits.CN16PUE = 0;	// disable pull-up resistor	RD7
	CNEN1bits.CN15IE = 1;	// enable pin change interrupt RD6
	CNPU1bits.CN15PUE = 0;	// disable pull-up resistor	RD6
	CNEN2bits.CN19IE = 1;	// enable pin change interrupt RD13
	CNPU2bits.CN19PUE = 0;	// disable pull-up resistor	RD13
	CNEN4bits.CN51IE = 1;	// enable pin change interrupt  RD2
	CNPU4bits.CN51PUE = 0;	// disable pull-up resistor	RD2
	CNEN4bits.CN50IE = 1;	// enable pin change interrupt RD1
	CNPU4bits.CN50PUE = 0;	// disable pull-up resistor RD1

	IFS1bits.CNIF = 0;		// clear IF clears flag
	IPC4bits.CNIP = 7;		// set IP as 7 priority level
	IEC1bits.CNIE = 1;		// enable CN
}



// PIN CHANGE INTERRUPT FOR USER INTERFACE 
////////////////////////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////////////////////////////////////
void __attribute__((interrupt, no_auto_psv)) _CNInterrupt(void)
{
	
	_temp16 = PORTD;		// read for next interrupt
	
	IFS1bits.CNIF = 0;		// clear IF
	__builtin_btg((unsigned int *)&LATA, 1);	// toggle the LED to indicate inerrupt
	
	
}

// INTERRUPT FUNCTION FOR UART RX COMMUNICATION
////////////////////////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////////////////////////////////////
void __attribute__ ((interrupt, no_auto_psv)) _U2RXInterrupt(void) 
{
	IFS1bits.U2RXIF = 0;	
	while( !OSCCONbits.COSC1);

	opt = U2RXREG;

	Nop();
	uflag = 1;
}
//}// U2RX interrupt vector
////////////////////////////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////////////////////////////////////////



// INITIALIZATOIN OF UART2
//////////////////////////////////////////////////////////////////////////////////////////////
//
//////////////////////////////////////////////////////////////////////////////////////////////

void InitUART2( void) 
{
	// The Explorer 16 board has a DB9 connector wired to UART2, so we will
	// be configuring this port only

	// configure U2MODE
	U2MODEbits.UARTEN = 0;			// Bit15 TX, RX DISABLED, ENABLE at end of func
	U2MODEbits.USIDL = 0;			// Bit13 Continue in Idle
	U2MODEbits.IREN = 0;			// Bit12 No IR translation
	U2MODEbits.RTSMD = 0;			// Bit11 Simplex Mode											
	U2MODEbits.UEN = 2;			// Bits8,9 TX,RX enabled, CTS,RTS not  							
	U2MODEbits.WAKE = 1;			// Bit7 Wake up (since we sleep here)
	U2MODEbits.LPBACK = 0;			// Bit6 No Loop Back
	U2MODEbits.ABAUD = 0;			// Bit5 No Autobaud (would require sending '55')
	U2MODEbits.RXINV = 0;			// Bit4 IdleState = 1
	U2MODEbits.BRGH = 1;			// Bit3 Fcy/4 OR 4 clocks per bit period  						 
	U2MODEbits.PDSEL = 0;			// Bits1,2 8bit, No Parity
	U2MODEbits.STSEL = 0;			// Bit0 One Stop Bit
	
	U2BRG = BAUDRATEREG2;			// baud rate

	// Load all values in for U1STA SFR
	U2STAbits.UTXISEL1 = 0;			//Bit15 Int when Char is transferred (1/2 config!)
	U2STAbits.UTXINV = 0;			//Bit14 N/A, IRDA config
	U2STAbits.UTXISEL0 = 0;			//Bit13 Other half of Bit15
	U2STAbits.UTXBRK = 0;			//Bit11 Disabled
	U2STAbits.UTXEN = 0;			//Bit10 TX pins controlled by periph
	U2STAbits.UTXBF = 0;			//Bit9 *Read Only Bit*
	U2STAbits.TRMT = 0;			//Bit8 *Read Only bit*
	U2STAbits.URXISEL = 0;			//Bits6,7 Int. on character recieved							
	U2STAbits.ADDEN = 0;			//Bit5 Address Detect Disabled
	U2STAbits.RIDLE = 0;			//Bit4 *Read Only Bit*
	U2STAbits.PERR = 0;			//Bit3 *Read Only Bit*
	U2STAbits.FERR = 0;			//Bit2 *Read Only Bit*
	U2STAbits.OERR = 0;			//Bit1 *Read Only Bit*
	U2STAbits.URXDA = 0;			//Bit0 *Read Only Bit*

	IEC1bits.U2TXIE = 0;			// Enable Transmit Interrupts
	IFS1bits.U2RXIF = 0;			// Clear the Recieve Interrupt Flag
	IEC1bits.U2RXIE = 1;			// Enable Recieve Interrupts

	U2MODEbits.UARTEN = 1;			// And turn the peripheral on

	U2STAbits.UTXEN = 1;

	IPC7bits.U2RXIP = 6;

}// end initialization of uart2
////////////////////////////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////////////////////////////////////////


// delay function
////////////////////////////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////////////////////////////////////////
void delay( int time)
{
	TMR1 = 0;
	T1CON = 0x8030;
	while( TMR1 < time);
	T1CON = 0;
}
// end delay function
////////////////////////////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////////////////////////////////////////
